package ssw.mj.codegen;

import static ssw.mj.Errors.Message.NO_ITEM;
import ssw.mj.Parser;
import ssw.mj.symtab.Obj;
import ssw.mj.symtab.Struct;
import ssw.mj.symtab.Tab;

/**
 * MicroJava Code Item. An item stores the attributes of an operand during code
 * generation.
 */
public class Item {
	/** Possible codes for the item kinds. */
	public enum Kind {
		Con, Local, Static, Stack, Fld, Elem, Meth, Cond
	}

	/** Kind of the item. */
	public Kind kind;
	/** The type of the item (reference to symbol table). */
	public Struct type;
	/** Only for Con: Value of the constant. */
	public int val;
	/** Only for Local, Static, Fld, Meth: Offset of the element. */
	public int adr;
	/** Only for Cond: Relational operator. */
	public int op;
	/** Only for Meth: Method object from the symbol table. */
	public Obj obj;
	/** Only for Cond: Target for true jumps. */
	public Label tLabel;
	/** Only for Cond: Target for false jumps. */
	public Label fLabel;

	public Item(Obj o, Parser parser) {
		type = o.type;
		val = o.val;
		adr = o.adr;
		switch (o.kind) {
			case Con:
				kind = Kind.Con;
				break;
			case Var:
				if (o.level == 0) {
					kind = Kind.Static;
				} else {
					kind = Kind.Local;
				}
				break;
			case Meth:
				kind = Kind.Meth;
				obj = o;
				break;
			default:
				parser.error(NO_ITEM);
		}
	}

	public Item(Kind kind, int x, Struct type, Parser parser) {
		this.kind = kind;
		this.type = type;
		switch (kind) {
			case Con:
				val = x;
				break;
			case Local:
			case Static:
			case Fld:
			case Meth:
				adr = x;
				break;
			case Cond:
				op = x;
				tLabel = new Label(parser.code);
				fLabel = new Label(parser.code);
				break;
			case Elem:
			case Stack:
				// Nothing to do
				break;
		}
	}

	/** Constructor for integer constants. */
	public Item(int x) {
		kind = Kind.Con;
		type = Tab.intType;
		val = x;
	}
}
