package ssw.mj.symtab;

/**
 * MicroJava Type Structures: A type structure stores the type attributes of a
 * declared type.
 */
public class Struct {
	/** Possible codes for structure kinds. */
	public enum Kind {
		None, Int, Char, Arr, Class
	}

	/** Kind of the structure node. */
	public Kind kind;
	/** Only for Arr: Type of the array elements. */
	public Struct elemType;
	/** Only for Class: Number of fields. */
	public int n;
	/** Only for Class: First element of the linked list of local variables. */
	public Obj fields;

	public Struct(Kind kind) {
		this.kind = kind;
	}

	public Struct(Kind kind, Struct elemType) {
		this(kind);
		if (kind == Kind.Arr) {
			this.elemType = elemType;
		}
	}

	// TODO Exercise 5: checks for different kinds of type compatibility

	/**
	 * String representation for debugging.
	 */
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		dump(sb, "", false);
		return sb.toString();
	}

	/**
	 * String representation for JUnit test cases.
	 */
	public void dump(StringBuilder sb, String indent, boolean dumpFields) {
		switch (kind) {
			case None:
				sb.append("void");
				break;
			case Int:
				sb.append("int");
				break;
			case Char:
				sb.append("char");
				break;
			case Arr:
				if (elemType != null) {
					elemType.dump(sb, indent, dumpFields);
				}
				sb.append("[]");
				break;
			case Class:
				sb.append("class(").append(n).append(")");
				if (dumpFields && fields != null) {
					sb.append("\n");
					fields.dump(sb, indent, true, true);
				}
				break;
		}
	}
}
